%% Information about the grid:

% 3 nodal grid: 110 kV
%       node 1:     generator   U1=110000/sqrt(3)  angle: 0 (correct voltages)
%       node 2:     generator   U2=110000/sqrt(3)  angle: 2 (correct voltages)
%       node 3:     load        U3=109000/sqrt(3)  angle: -4 (correct voltages)
%       line 12:                54.881km
%       line 13:                62.198km   
%       r'= 0.08 Ohm/km
%       l'= 1.206e-3 H/km

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% PREPARATION SECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Section 0: Searched variables
% Task 0)**************************************************************+
syms %?
f1  %=?

%% Section 1: Measurements
% Measurement Values
%P1 = 5.267e6;
%Q1 = 0.975e6;
P2 = 6.457e6;
Q2 = -1.482e6;
P3 = -11.724e6;
Q3 = 0.506e6;
P13 = 11.725e6;
Q13 = -0.506e6;
P12 = -6.457e6;
Q12 = 1.482e6;
%U1 = 1/sqrt(3)*110e3;
U2 = 1/sqrt(3)*110e3;
U3 = 1/sqrt(3)*109e3;

%% Section 2: Line parameters
% Line Parameters (cross elements can be neglected)
%       line 12:                54.881km
%       line 13:                62.198km   
%       r'= 0.08 Ohm/km
%       l'= 1.206e-3 H/km
% longitudinal elements
Z12 = 0.08  + 1i*54.881*1.206e-3 *2*pi()*50;
Z13 = 0.08  + 1i*1.206e-3*62.198 *2*pi()*50;
% Task 2) conductance G (two port line element) **********************
G11_2 = real(1/Z12); 
G22_1 = G11_2;
G12 = real(-1/Z12); 
G21 = G12;
G11_3 %=? 
G33_1 %=?
G13 %=?
G31 %=?
% Task 2) susceptance B (two port line element)***********************
B11_2 = imag(1/Z12); 
B22_1 = B11_2;
B12 %=? 
B21 %=?
B11_3 %=?
B33_1 %=?
B13 %=?
B31 %=?

%% Section 3: grid equations
% Task 3) Power flow equations, nodal power equations and voltage
% equations ****************************************
% power flow
fP12 = (e1^2+f1^2)*G11_2 + (e1*e2+f1*f2)*G12 - (e1*f2-e2*f1)*B12;
fP13 %=?
fQ12 %=?
fQ13 %=?
% nodal power
fP1 = fP12 + fP13; 
fP2 = -fP12;
fP3 %=? 
fQ1 %=?
fQ2%=?
fQ3%=?
% voltages
fU1 %=?
fU2 %=?
fU3 %=?

%% Section 4: Derivation
% Task 4) Derivation ***********************************************
dP12f2 = diff(fP12,f2);
dP12f3 = diff(fP12,f3);
dP12e1 = diff(fP12,e1);
dP12e2 = diff(fP12,e2);
dP12e3 = diff(fP12,e3);
rowP12 = [dP12f2,dP12f3,dP12e1,dP12e2,dP12e3];

dP13f2 %=?
dP13f3 %=?
dP13e1 %=?
dP13e2 %=?
dP13e3 %=?
rowP13 %=?

dQ12f2 %=?
dQ12f3 %=?
dQ12e1 %=?
dQ12e2 %=?
dQ12e3 %=?
rowQ12 %=?

dQ13f2 = diff(fQ13,f2);
dQ13f3 = diff(fQ13,f3);
dQ13e1 = diff(fQ13,e1);
dQ13e2 = diff(fQ13,e2);
dQ13e3 = diff(fQ13,e3);
rowQ13 = [dQ13f2,dQ13f3,dQ13e1,dQ13e2,dQ13e3];


dP1f2 = diff(fP1,f2);
dP1f3 = diff(fP1,f3);
dP1e1 = diff(fP1,e1);
dP1e2 = diff(fP1,e2);
dP1e3 = diff(fP1,e3);
rowP1 = [dP1f2,dP1f3,dP1e1,dP1e2,dP1e3];

dP2f2 = diff(fP2,f2);
dP2f3 = diff(fP2,f3);
dP2e1 = diff(fP2,e1);
dP2e2 = diff(fP2,e2);
dP2e3 = diff(fP2,e3);
rowP2 = [dP2f2,dP2f3,dP2e1,dP2e2,dP2e3];

dP3f2 = diff(fP3,f2);
dP3f3 = diff(fP3,f3);
dP3e1 = diff(fP3,e1);
dP3e2 = diff(fP3,e2);
dP3e3 = diff(fP3,e3);
rowP3 = [dP3f2,dP3f3,dP3e1,dP3e2,dP3e3];

dQ1f2 = diff(fQ1,f2);
dQ1f3 = diff(fQ1,f3);
dQ1e1 = diff(fQ1,e1);
dQ1e2 = diff(fQ1,e2);
dQ1e3 = diff(fQ1,e3);
rowQ1 = [dQ1f2,dQ1f3,dQ1e1,dQ1e2,dQ1e3];

dQ2f2 = diff(fQ2,f2);
dQ2f3 = diff(fQ2,f3);
dQ2e1 = diff(fQ2,e1);
dQ2e2 = diff(fQ2,e2);
dQ2e3 = diff(fQ2,e3);
rowQ2 = [dQ2f2,dQ2f3,dQ2e1,dQ2e2,dQ2e3];

dQ3f2 = diff(fQ3,f2);
dQ3f3 = diff(fQ3,f3);
dQ3e1 = diff(fQ3,e1);
dQ3e2 = diff(fQ3,e2);
dQ3e3 = diff(fQ3,e3);
rowQ3 = [dQ3f2,dQ3f3,dQ3e1,dQ3e2,dQ3e3];


dU1f2 = diff(fU1,f2);
dU1f3 = diff(fU1,f3);
dU1e1 = diff(fU1,e1);
dU1e2 = diff(fU1,e2);
dU1e3 = diff(fU1,e3);
rowU1 = [dU1f2,dU1f3,dU1e1,dU1e2,dU1e3];

dU2f2 = diff(fU2,f2);
dU2f3 = diff(fU2,f3);
dU2e1 = diff(fU2,e1);
dU2e2 = diff(fU2,e2);
dU2e3 = diff(fU2,e3);
rowU2 = [dU2f2,dU2f3,dU2e1,dU2e2,dU2e3];

dU3f2 = diff(fU3,f2);
dU3f3 = diff(fU3,f3);
dU3e1 = diff(fU3,e1);
dU3e2 = diff(fU3,e2);
dU3e3 = diff(fU3,e3);
rowU3 = [dU3f2,dU3f3,dU3e1,dU3e2,dU3e3];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% STATE ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Section 5: z, h(x), H(x), R_inv
% Task 5 a,b,c,d: Measurement vector z, equation vector h(x), Jacobian matrix H(x)
% Standard deviation matrix R_inv ********************************************
z %=?
h(f2,f3,e1,e2,e3)%=?
H(f2,f3,e1,e2,e3)  %=?
R_inv %=?

%% Section 6: Iteration Algorithm
% Start values
x_k = [0,0,1/sqrt(3)*110e3,1/sqrt(3)*110e3,1/sqrt(3)*110e3].';
epsilon = 10; % start value
k = 0; % iteration step
r = 25; % number of decimal places

% save information about x_k in every iteration (necessary for plots)
U_calculated = [x_k(3), x_k(4)+1i*x_k(1), x_k(5)+1i*x_k(2)]; % complex voltages U1, U2, U3

while abs(epsilon) > 10e-10 && k < 150
   % print the iteration step to the command window
   k

   % Task 6a) calculate Hxk ********************************************
   Hxk%=?
    
   % Task 6b) calculate G and round G and invert G *********************
   G %=?
   G %=?
   invG %=?
   
   % Task 6c) calculate h(x) and improve x **************************
   hxk %=?
   x_k1 %=?

   % convergence
   epsilon = norm(x_k1-x_k);
   
   % increase iteration step and set improved x
   k = k+1;
   x_k = x_k1;
    
   % save information about x_k in every iteration (necessary for plots)
   U_calculated = [U_calculated; x_k(3), x_k(4)+1i*x_k(1), x_k(5)+1i*x_k(2)]; % complex voltages U1, U2, U3

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% EVALUATION SECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Command Window and Plots
% correct voltage
Uk = (1/sqrt(3))*[110e3*exp(0i);110e3*exp(2i*pi()/180);109e3*exp(-4i*pi()/180)];
U_abs_correct = round(abs(Uk),2);
U_ang_correct = round(angle(Uk)*180/pi(), 2);
% calculated
U_calc =double([vpa(x_k(3)+1i*f1); vpa(x_k(4)+1i*x_k(1)); vpa(x_k(5)+1i*x_k(2))]);
U_abs_calc = round(abs(U_calc),2);
U_ang_calc = round(angle(U_calc)*180/pi(), 2);
nodes = [1; 2; 3];
%ToPrint = table(nodes,Uk, U_abs,U_ang);
ToPrint = table(nodes,U_abs_correct, U_ang_correct, U_abs_calc, U_ang_calc);
disp(ToPrint)

% plotting
plotting = 1
if plotting == 1
    U1_calculated = double(U_calculated(:,1))./1000;
    U2_calculated = double(U_calculated(:,2))./1000;
    U3_calculated = double(U_calculated(:,3))./1000;

    % correct voltage
    k = 1:1:length(U1_calculated);
    U1_correct = ones(length(k),1)*Uk(1)./1000;
    U2_correct = ones(length(k),1)*Uk(2)./1000;
    U3_correct = ones(length(k),1)*Uk(3)./1000;

        % Betrge U1, U2, U3
    figure
    plot(k,abs(U1_correct),'Color',"#4DBEEE",'LineWidth',2)
    hold on
    plot(k,abs(U3_correct),'Color',	"#EDB120",'LineWidth',2)
    hold on    
    plot(k,abs(U1_calculated),'--','Color', "#0000FF",'LineWidth',2) 
    hold on
    plot(k,abs(U2_calculated),'--','Color',"#0072BD",'LineWidth',2)
    hold on
    plot(k,abs(U3_calculated),'--','Color',	"#D95319",'LineWidth',2)
    legend('U1, U2 correct','U3 correct','U1','U2', 'U3', 'FontSize',20)
    xlabel('iterations k','FontSize',20)
    ylabel('U Absolute [kV]','FontSize',20)
    set(gca,"FontSize",20)
    ylim([61 64])

    % Angle U2, U3
    figure
    plot(k,angle(U2_correct)*180/pi(),'Color',"#4DBEEE",'LineWidth',2)
    hold on
    plot(k,angle(U3_correct)*180/pi(),'Color',	"#EDB120",'LineWidth',2)
    hold on
    plot(k,angle(U2_calculated)*180/pi(),'--','Color',"#0072BD",'LineWidth',2) 
    hold on
    plot(k,angle(U3_calculated)*180/pi(),'--','Color',	"#D95319",'LineWidth',2) 
    legend('U2.ang correct','U3.ang correct',  'U2.ang', 'U3.ang','FontSize',20)
    xlabel('iterations k','FontSize',20)
    ylabel('U Angle [Grad]','FontSize',20)
    set(gca,"FontSize",20)
end



